<?php namespace Dever;
use Dever;
class Debug
{
    public static $shell;
    private static $data;
    private static $trace;
    private static $time;
    private static $memory;
    private static $start;
    public static function init()
    {
        ini_set('display_errors', true);
        self::$start = microtime();
        set_error_handler(array(__CLASS__, 'error_handler'), E_ERROR | E_NOTICE | E_STRICT);
        set_exception_handler(array(__CLASS__, 'exception_handler'));
        self::$shell = Dever::shell(Config::get('setting')['shell']);
    }
    public static function error_handler($no, $str, $file, $line)
    {
        self::$shell = true;
        $data['msg'] = $str;
        $data['file'] = $file . ':' . $line;
        $content = file_get_contents($file);
        $lines = explode("\n", $content);
        $lines = array_slice($lines, $line - 10, 20, true);
        $code = '<code>';
        foreach ($lines as $k => $v) {
            $k+=1;
            if ($k == $line) {
                $code .= '<a style="display:inline-block;color:red">' . $k . $v . '</a>';
            } else {
                $code .= $k . $v;
            }
            $code .= "\n";
        }
        $code .= '</code>';
        self::add($code, false);
        self::out($data, false);
    }
    public static function exception_handler($exception)
    {
        self::error_handler($exception->getCode(), $exception->getMessage(), $exception->getFile(), $exception->getLine());
    }
    public static function add($data, $type = 'log')
    {
        if (self::$shell) {
            $msg = $data;
            if ($type) {
                $msg = is_array($msg) ? $msg : array('msg' => $msg);
                $msg = array_merge($msg, self::env());
            }
            self::data($type, Output::format($msg));
        }
        return $data;
    }
    public static function out($data, $type = 'out')
    {
        if (self::$shell) {
            self::add($data, $type);
            self::total();
            echo self::html();die;
        }
    }
    public static function total()
    {
        self::data('include', Output::format(self::load()));
        $msg = array('time' => self::time(), 'memory' => self::memory());
        self::data('total', Output::format($msg));
        return $msg;
    }
    public static function lib($class, $method)
    {
        if (self::$shell) {
            $class = new \ReflectionClass($class);
            $trace['file'] = $class->getFileName();
            $trace['line'] = $class->getStartLine();
            $trace['class'] = $class->getName();
            $trace['function'] = $method;
            $content = explode("\n", file_get_contents($trace['file']));
            foreach ($content as $k => $v) {
                if (strpos($v, 'function ' . $method . '(')) {
                    $trace['line'] = $k+1;
                    break;
                }
            }
            $key = $trace['file'] . ':' . $trace['line'];
            self::$trace[$key] = $trace;
        }
    }
    public static function trace()
    {
        $debug = debug_backtrace();
        $trace = '';
        if ($debug) {
            foreach($debug as $k => $v) {
                if (self::check($v)) {
                    $trace = $v['file'] . ':' . $v['line'];
                    self::$trace[$trace] = $v;
                    break;
                }
            }
        }
        if (!$trace && self::$trace) {
            $trace = array_keys(self::$trace);
            $trace = $trace[0];
        }
        return $trace;
    }
    public static function getTrace()
    {
        if (self::$trace) {
            return array_reverse(array_values(self::$trace));
        }
        return array();
    }
    private static function env()
    {
        $trace = self::trace();
        return array
        (
            'time' => 'current:' . self::time(2) . ' total:' . self::time(),
            'memory' => 'current:' . self::memory(2) . ' total:' . self::memory(),
            'trace' => $trace
        );
    }
    private static function data($method, $msg)
    {
        if (!$method) {
            $method = 'error';
        }
        self::$data[$method][] = $msg;
    }
    private static function check($value)
    {
        if (isset($value['file']) && strpos($value['file'], DEVER_APP_PATH) !== false) {
            $config = array('lib', 'api', 'table');
            foreach ($config as $k => $v) {
                if (strpos($value['file'], DEVER_APP_PATH . $v) !== false) {
                    return true;
                }
            }
        }
        return false;
    }
    private static function time($state = 1)
    {
        $start = self::startTime($state);
        $end = self::endTime();
        return '[' . ($end - $start) . 'S]';
    }
    private static function endTime()
    {
        self::$time = microtime();
        return self::createTime(self::$time);
    }
    private static function startTime($state = 1)
    {
        $start = self::$start;
        if ($state == 2 && self::$time) {
            $start = self::$time;
        }
        return self::createTime($start);
    }
    private static function createTime($time)
    {
        list($a, $b) = explode(' ', $time);
        return ((float) $a + (float) $b);
    }
    private static function memory($state = 1)
    {
        $memory = memory_get_usage();
        if ($state == 2 && self::$memory) {
            $memory = $memory - self::$memory;
        }
        self::$memory = $memory;
        return '[' . ($memory / 1024) . 'KB]';
    }
    private static function load()
    {
        $files = get_included_files();
        $result = array();
        $path = DEVER_PATH;
        foreach ($files as $k => $v) {
            if (strpos($v, $path) === false) {
                $result[] = $v;
            }
        }
        return $result;
    }
    private static function html($show = '')
    {
        $html = '<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.6.0/styles/atom-one-dark.min.css">
<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.6.0/highlight.min.js"></script><div>';
        if (self::$data) {
            foreach (self::$data as $k => $v) {
                $html .= $k;
                $html .= '<table style="width:100%;">';
                foreach ($v as $i => $j) {
                    $html .= '<tr>';
                    $html .= '<td style="background-color:#f5f5f5;padding: 8px;">' . $j . '</td>';
                    $html .= '</tr>';
                }
                $html .= '</table>';
            }
        }
        $html .= '</div><script>hljs.highlightAll();</script>';
        return $html;
    }
}