<?php namespace Dever\Helper;
use Dever\Debug;
class Redis
{
    private static $handle;
    private static $expire = 3600;
    public static function connect()
    {
        if (!self::$handle) {
            $config = \Dever::config('setting')['redis'] ?? false;
            if (!$config) {
                \Dever::error('redis error');
            }
            self::$expire = $config['expire'];
            self::$handle = new \Redis;
            self::$handle->pconnect($config["host"], $config["port"]);
            if (isset($config['password']) && $config['password']) {
                self::$handle->auth($config['password']);
            }
            self::$handle->setOption(\Redis::OPT_READ_TIMEOUT, -1);
        }
        return self::$handle;
    }
    public static function get($key)
    {
        return Debug::add(self::connect()->get($key), 'cache');
    }
    public static function set($key, $value, $expire = 0)
    {
        self::expire($expire);
        return self::connect()->set($key, $value, self::$expire);
    }
    public static function lock($key, $value, $expire = 0)
    {
        self::expire($expire);
        return self::connect()->set($key, $value, ['NX', 'EX' => $expire]);
    }
    public static function unlock($key, $value)
    {
        $script = <<< EOF
        if redis.call("get", KEYS[1]) == ARGV[1] then
            return redis.call("del", KEYS[1])
        else
            return 0
        end
        EOF;
        return self::connect()->eval($script, [$key, $value], 1);    
    }
    public static function incr($key, $value = false)
    {
        if ($value) {
            return self::connect()->incrby($key, $value);
        }
        return self::connect()->incr($key);
    }
    public static function decr($key, $value = false)
    {
        if ($value) {
            return self::connect()->decrby($key, $value);
        }
        return self::connect()->decr($key);
    }
    public static function push($key, $value)
    {
        return self::connect()->lpush($key, $value);
    }
    public static function pop($key)
    {
        $data = self::connect()->brPop($key, 10);
        if ($data) {
            if (isset($data[1])) {
                $data = $data[1];
            }
            return $data;
        } else {
            $pong = self::connect()->ping();
            if ($pong != '+PONG') {
                throw new \Exception('Redis ping failure!', 500);
            }
            usleep(100000);
        }
        return false;
    }
    public static function len($key)
    {
        return self::connect()->llen($key);
    }
    public static function hGet($key, $hkey = false)
    {
        if ($hkey) {
            return Debug::add(self::connect()->hGet($key, $hkey), 'cache');
        } else {
            return Debug::add(self::connect()->hGetAll($key), 'cache');
        }
    }
    public static function hDel($key, $hkey)
    {
        return self::connect()->hDel($key, $hkey);
    }
    public static function hExists($key, $hkey)
    {
        return self::connect()->hExists($key, $hkey);
    }
    public static function hKeys($key)
    {
        return self::connect()->hKeys($key);
    }
    public static function hSet($key, $hkey, $value, $expire = 0)
    {
        self::expire($expire);
        return self::connect()->hKeys($key, $hkey, $value, self::$expire);
    }
    public static function delete($key)
    {
        return self::connect()->delete($key, 0);
    }
    public static function close()
    {
        return self::connect()->close();
    }
    private static function expire($expire)
    {
        if ($expire) {
            self::$expire = $expire;
        }
    }
}