<?php namespace Dever;
use Dever;
class Output
{
    private static $format = 'json';
    public static function success($data, $uuid = false, $code = 200)
    {
        $result = array();
        $result['status'] = 1;
        $result['msg'] = 'success';
        $result['data'] = $data;
        $result['code'] = $code;
        if ($page = Paginator::get()) {
            $result['page'] = $page;
        }
        if ($uuid) {
            $result['uuid'] = Dever::uuid();
        }
        self::handle($result);
        return $data;
    }
    public static function error($msg, $code = 500)
    {
        $result = array();
        $result['status'] = 2;
        $result['code'] = $code;
        $result['msg'] = $msg;
        self::handle($result);
        die;
    }
    public static function out($result)
    {
        return self::handle($result);
    }
    public static function format($data)
    {
        if (is_array($data)) {
            $data = htmlspecialchars(json_encode($data, JSON_PRETTY_PRINT + JSON_UNESCAPED_SLASHES + JSON_UNESCAPED_UNICODE));
        }
        $content = "<pre>\n";
        $content .= $data;
        $content .= "\n</pre>\n";
        return $content;
    }
    public static function handle(&$result)
    {
        if (isset(Dever::config('setting')['output'])) {
            foreach (Dever::config('setting')['output'] as $k => $v) {
                if (isset($result[$k])) {
                    if (is_array($v)) {
                        $result[$v[0]] = $v[1][$result[$k]] ?? $result[$k];
                    } else {
                        $result[$v] = $result[$k];
                    }
                    unset($result[$k]);
                }
            }
        }
        Debug::out($result);
        self::json($result);
        self::callback($result);
        self::func($result);
        if (self::$format == 'json') {
            print_r($result);
        } else {
            self::html($result);
        }
    }
    private static function json(&$result)
    {
        if (self::$format == 'json' || Route::input('json') == 1) {
            if (!$result) {
                $result = (object) $result;
            }
            $result = Dever::json_encode($result);
            self::$format = 'json';
        } else {
            self::$format = 'str';
        }
    }
    private static function callback(&$result)
    {
        if ($callback = Route::input('callback')) {
            $result = $callback . '(' . $result . ')';
        }
    }
    private static function func(&$result)
    {
        if ($function = Route::input('function')) {
            $result = '<script>parent.' . $function . '(' . $result . ')' . '</script>';
        }
    }
    public static function html($msg)
    {
        $html = '' . $msg['msg'];
        $host = Route::url('');
        $name = '404';
        if ($msg['code'] > 1) {
            $name = $msg['code'];
        }
        if ($name == 404) {
            header("HTTP/1.1 404 Not Found");
            header("Status: 404 Not Found");
        }
        $file = DEVER_APP_PATH . 'config/html/' . $name . '.html';
        if (is_file($file)) {
            include $file;
        } else {
            $file = DEVER_PROJECT_PATH . 'config/html/' . $name . '.html';
            if (is_file($file)) {
                include $file;
            } else {
                include DEVER_PATH . 'config/html/default.html';
            }
        }
    }
}